'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _errors = require('./errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Binds a namespace
 * @param {string} namespace
 * @param {Object} options
 * @param {Object} options.data
 * @private
 * @returns {Promise<Object>}
 */
function _bind(namespace) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  options = options || {};
  if (!namespace) {
    return _promise2.default.reject(new Error('`namespace` is required'));
  }

  if (!options.logger) {
    return _promise2.default.reject(new Error('`options.logger` is required'));
  }

  var _options = options,
      logger = _options.logger;


  var map = new _map2.default([['@', {}]]);

  if (options.data) {
    (0, _keys2.default)(options.data).forEach(function (key) {
      map.set(key, options.data[key]);
    });
  }

  logger.debug('memory-store-adapter: returning binding');

  return _promise2.default.resolve({
    clear: function clear() {
      logger.debug('memory-store-adapter: clearing the binding');

      return _promise2.default.resolve(map.clear());
    },
    del: function del(key) {
      logger.debug('memory-store-adapter: deleting `' + key + '`');

      return _promise2.default.resolve(map.delete(key));
    },
    get: function get(key) {
      logger.debug('memory-store-adapter: reading `' + key + '`');
      var res = map.get(key);

      if (typeof res === 'undefined') {
        return _promise2.default.reject(new _errors.NotFoundError());
      }

      return _promise2.default.resolve(res);
    },
    put: function put(key, value) {
      logger.debug('memory-store-adapter: writing `' + key + '`');

      return _promise2.default.resolve(map.set(key, value));
    }
  });
} /*!
   * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
   */

exports.default = {
  bind: _bind,
  preload: function preload(data) {
    return {
      bind: function bind(namespace) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        if (data[namespace]) {
          options.data = data[namespace];
        }

        return _bind(namespace, options);
      }
    };
  }
};
//# sourceMappingURL=memory-store-adapter.js.map
