import * as chrono from '../../src/chrono';
import { testSingleCase } from '../test_util';

test("Test - Single Expression", function() {

    testSingleCase(chrono.es, 'hace 5 días, hicimos algo', new Date(2012,7,10), (result) => {
        expect(result.start).not.toBeNull()
        expect(result.start.get('year')).toBe(2012)
        expect(result.start.get('month')).toBe(8)
        expect(result.start.get('day')).toBe(5)

        expect(result.index).toBe(0)
        expect(result.text).toBe('hace 5 días')

        var resultDate = result.start.date();
        var expectDate = new Date(2012, 8-1, 5, 12);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });

    testSingleCase(chrono.es, 'hace 10 dias, hicimos algo', new Date(2012,7,10), (result) => {
        expect(result.start).not.toBeNull()
        expect(result.start.get('year')).toBe(2012)
        expect(result.start.get('month')).toBe(7)
        expect(result.start.get('day')).toBe(31)

        expect(result.index).toBe(0)
        expect(result.text).toBe('hace 10 dias')

        var resultDate = result.start.date();
        var expectDate = new Date(2012, 7-1, 31, 12);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });


    testSingleCase(chrono.es, 'hace 15 minutos', new Date(2012,7,10,12,14), (result) => {
        expect(result.index).toBe(0)
        expect(result.text).toBe('hace 15 minutos')
        expect(result.start.get('hour')).toBe(11)
        expect(result.start.get('minute')).toBe(59)

        var resultDate = result.start.date();
        var expectDate = new Date(2012,7,10,11,59);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });

    testSingleCase(chrono.es, '   hace 12 horas', new Date(2012,7,10,12,14), (result) => {
        expect(result.index).toBe(3)
        expect(result.text).toBe('hace 12 horas')
        expect(result.start.get('hour')).toBe(0)
        expect(result.start.get('minute')).toBe(14)

        var resultDate = result.start.date();
        var expectDate = new Date(2012,7,10,0,14);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });
});


test("Test - Single Expression (Casual)", function() {

    testSingleCase(chrono.es, 'hace 5 meses, hicimos algo', new Date(2012, 8-1,10), (result) => {
        expect(result.start).not.toBeNull()
        expect(result.start.get('year')).toBe(2012)
        expect(result.start.get('month')).toBe(3)
        expect(result.start.get('day')).toBe(10)

        expect(result.index).toBe(0)
        expect(result.text).toBe('hace 5 meses')

        var resultDate = result.start.date();
        var expectDate = new Date(2012, 3-1, 10, 12);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });

    testSingleCase(chrono.es, 'hace 5 años, hicimos algo', new Date(2012, 8-1,10), (result) => {
        expect(result.start).not.toBeNull()
        expect(result.start.get('year')).toBe(2007)
        expect(result.start.get('month')).toBe(8)
        expect(result.start.get('day')).toBe(10)

        expect(result.index).toBe(0)
        expect(result.text).toBe('hace 5 años')

        var resultDate = result.start.date();
        var expectDate = new Date(2007, 8-1, 10, 12);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });


    testSingleCase(chrono.es, 'hace una semana, hicimos algo', new Date(2012, 8-1, 3), (result) => {
        expect(result.start).not.toBeNull()
        expect(result.start.get('year')).toBe(2012)
        expect(result.start.get('month')).toBe(7)
        expect(result.start.get('day')).toBe(27)

        expect(result.index).toBe(0)
        expect(result.text).toBe('hace una semana')

        var resultDate = result.start.date();
        var expectDate = new Date(2012, 7-1, 27, 12);
        expect(expectDate.getTime()).toBeCloseTo(resultDate.getTime())
    });
});
